@@ -719,9 +719,9 @@ class Client(object):
                     yield entry
 
                 if self._is_dir(fileinfo.fs) and (include_children or recurse):
-                    listing = self._get_dir_listing(path)
-                    for node in listing.dirList.partialListing:
+                    for node in self._get_dir_listing(path):
                         full_path = self._get_full_path(path, node)
+                        last_entry_path = node.path
                         entry = processor(full_path, node)
                         yield entry
 
@@ -736,12 +736,20 @@ class Client(object):
                                                          recurse=recurse):
                                 yield item
 
-    def _get_dir_listing(self, path):
+    def _get_dir_listing(self, path, start_after=''):
         request = client_proto.GetListingRequestProto()
         request.src = path
-        request.startAfter = ''
+        request.startAfter = start_after
         request.needLocation = False
-        return self.service.getListing(request)
+        listing = self.service.getListing(request)
+        if not listing:
+            return
+        for node in listing.dirList.partialListing:
+            start_after = node.path
+            yield node
+        if listing.dirList.remainingEntries > 0:
+            for node in self._get_dir_listing(path, start_after):
+                yield node
 
     def _glob_find(self, path, processor, include_toplevel):
         '''Handle globs in paths.
@@ -750,6 +758,11 @@ class Client(object):
         we don't add the found children to the result, but traverse into paths
         that did have a match.
         '''
+
+        # Remove the last / from the path, since hadoop doesn't understand it
+        if path.endswith("/"):
+            path = path[:-1]
+
         # Split path elements and check where the first occurence of magic is
         path_elements = path.split("/")
         for i, element in enumerate(path_elements):
@@ -772,16 +785,14 @@ class Client(object):
         # we prepend it with "/", since "/".join(['x']) doesn't return "/x"
         rest_elements = path_elements[first_magic + 1:]
         if len(rest_elements) == 1:
-            rest = "/" + rest_elements[0]
+            rest = rest_elements[0]
         else:
             rest = "/".join(rest_elements)
-
         # Check if the path exists and that it's a directory (which it should..)
         fileinfo = self._get_file_info(check_path)
         if fileinfo and self._is_dir(fileinfo.fs):
             # List all child nodes and match them agains the glob
-            listing = self._get_dir_listing(check_path)
-            for node in listing.dirList.partialListing:
+            for node in self._get_dir_listing(check_path):
                 full_path = self._get_full_path(check_path, node)
                 if fnmatch.fnmatch(full_path, match_path):
                     # If we have a match, but need to go deeper, we recurse
@@ -789,6 +800,16 @@ class Client(object):
                         traverse_path = "/".join([full_path, rest])
                         for item in self._glob_find(traverse_path, processor, include_toplevel):
                             yield item
+                    elif rest:
+                        # we have more rest, but it's not magic, which is either a file or a directory
+                        final_path = os.path.join(full_path, rest)
+                        fi = self._get_file_info(final_path)
+                        if fi and self._is_dir(fi.fs):
+                            for n in self._get_dir_listing(final_path):
+                                full_child_path = self._get_full_path(final_path, n)
+                                yield processor(full_child_path, n)
+                        elif fi:
+                            yield processor(final_path, fi.fs)
                     else:
                         # If the matching node is a directory, we list the directory
                         # This is what the hadoop client does at least.
@@ -798,7 +819,7 @@ class Client(object):
                             fp = self._get_full_path(check_path, node)
                             dir_list = self._get_dir_listing(fp)
                             if dir_list:  # It might happen that the directory above has been removed
-                                for n in dir_list.dirList.partialListing:
+                                for n in dir_list:
                                     full_child_path = self._get_full_path(fp, n)
                                     yield processor(full_child_path, n)
                         else:
